# Copyright: (c) 2019, Olivier Blin <olivier.oblin@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from units.compat.mock import patch
from ansible.modules.network.nxos import nxos_l3_interface
from .nxos_module import TestNxosModule, load_fixture, set_module_args


class TestNxosL3InterfaceModule(TestNxosModule):

    module = nxos_l3_interface

    def setUp(self):
        super(TestNxosL3InterfaceModule, self).setUp()

        self.mock_load_config = patch('ansible.modules.network.nxos.nxos_l3_interface.load_config')
        self.load_config = self.mock_load_config.start()

        self.mock_get_config = patch('ansible.modules.network.nxos.nxos_l3_interface.get_config')
        self.get_config = self.mock_get_config.start()

    def tearDown(self):
        super(TestNxosL3InterfaceModule, self).tearDown()
        self.mock_load_config.stop()
        self.mock_get_config.stop()

    def load_fixtures(self, commands=None, device=''):
        self.load_config.return_value = None
        self.get_config.return_value = load_fixture('nxos_l3_interface', self.mode)

    def test_nxos_l3_interface_unknonw_ethernet(self):
        self.mode = 'ethernet_noshut'
        set_module_args(dict(name='Ethernet1/2', ipv4='192.168.0.1/24'))
        result = self.execute_module(changed=False)

    # Add when missing
    def test_nxos_l3_interface_add_missing_ipv4(self):
        self.mode = 'ethernet_noshut'
        set_module_args(dict(name='Ethernet1/1', ipv4='192.168.0.1/24'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['interface Ethernet1/1', 'ip address 192.168.0.1/24', 'exit'])

    def test_nxos_l3_interface_add_missing_ipv4_on_e11(self):
        self.mode = 'ethernet_noshut'
        set_module_args(dict(name='et1/1', ipv4='192.168.0.1/24'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['interface Ethernet1/1', 'ip address 192.168.0.1/24', 'exit'])

    def test_nxos_l3_interface_add_missing_ipv6(self):
        self.mode = 'ethernet_noshut'
        set_module_args(dict(name='Ethernet1/1', ipv6='2001:db8::1/124'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['interface Ethernet1/1', 'ipv6 address 2001:db8::1/124', 'exit'])

    def test_nxos_l3_interface_add_missing_ipv4_and_ipv6(self):
        self.mode = 'ethernet_noshut'
        set_module_args(dict(name='Ethernet1/1', ipv4='192.168.0.1/24', ipv6='2001:db8::1/124'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['interface Ethernet1/1', 'ip address 192.168.0.1/24', 'ipv6 address 2001:db8::1/124', 'exit'])

    # Add when existing
    def test_nxos_l3_interface_add_existing_ipv4(self):
        self.mode = 'ethernet_noshut_ipv4_ipv6'
        set_module_args(dict(name='Ethernet1/1', ipv4='192.168.0.1/24'))
        result = self.execute_module()

    def test_nxos_l3_interface_add_existing_ipv4_on_e11(self):
        self.mode = 'ethernet_noshut_ipv4_ipv6'
        set_module_args(dict(name='et1/1', ipv4='192.168.0.1/24'))
        result = self.execute_module()

    def test_nxos_l3_interface_add_existing_ipv6(self):
        self.mode = 'ethernet_noshut_ipv4_ipv6'
        set_module_args(dict(name='Ethernet1/1', ipv6='2001:db8::1/124'))
        result = self.execute_module()

    def test_nxos_l3_interface_add_existing_ipv4_and_ipv6(self):
        self.mode = 'ethernet_noshut_ipv4_ipv6'
        set_module_args(dict(name='Ethernet1/1', ipv4='192.168.0.1/24', ipv6='2001:db8::1/124'))
        result = self.execute_module()

    def test_nxos_l3_interface_new_ipv4_and_ipv6(self):
        self.mode = 'ethernet_noshut_ipv4_ipv6'
        set_module_args(dict(name='Ethernet1/1', ipv4='192.168.0.2/24', ipv6='2001:db8::2/124'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['interface Ethernet1/1', 'ip address 192.168.0.2/24', 'ipv6 address 2001:db8::2/124', 'exit'])

    # Add when existing with multiple IPv6
    def test_nxos_l3_interface_multiple_ipv6_add_first(self):
        self.mode = 'ethernet_noshut_multiple_ipv6'
        set_module_args(dict(name='Ethernet1/1', ipv6='2001:db8::1/124'))
        result = self.execute_module()

    def test_nxos_l3_interface_multiple_ipv6_add_last(self):
        self.mode = 'ethernet_noshut_multiple_ipv6'
        set_module_args(dict(name='Ethernet1/1', ipv6='2001:db8:2::1/124'))
        result = self.execute_module()

    # Add aggregate
    def test_nxos_l3_interface_add_missing_with_empty_aggregate(self):
        self.mode = 'ethernet_noshut'
        set_module_args(dict(aggregate=[]))
        result = self.execute_module()

    def test_nxos_l3_interface_add_missing_with_aggregate(self):
        self.mode = 'ethernet_noshut'
        set_module_args(dict(aggregate=[
            dict(name='Ethernet1/1', ipv4='192.168.0.2/24', ipv6='2001:db8::2/124'),
            dict(name='Ethernet1/1', ipv6='2001:db8:1::2/124'),
            dict(name='Ethernet1/1', ipv6='2001:db8:2::2/124')]))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], [
            'interface Ethernet1/1', 'ip address 192.168.0.2/24', 'ipv6 address 2001:db8::2/124', 'exit',
            'interface Ethernet1/1', 'ipv6 address 2001:db8:1::2/124', 'exit',
            'interface Ethernet1/1', 'ipv6 address 2001:db8:2::2/124', 'exit'])

    # Rem when missing
    def test_nxos_l3_interface_rem_missing_ipv4(self):
        self.mode = 'ethernet_noshut'
        set_module_args(dict(name='Ethernet1/1', ipv4='192.168.0.1/24', state='absent'))
        result = self.execute_module()

    def test_nxos_l3_interface_rem_missing_ipv4_on_e11(self):
        self.mode = 'ethernet_noshut'
        set_module_args(dict(name='et1/1', ipv4='192.168.0.1/24', state='absent'))
        result = self.execute_module()

    def test_nxos_l3_interface_rem_missing_ipv6(self):
        self.mode = 'ethernet_noshut'
        set_module_args(dict(name='Ethernet1/1', ipv6='2001:db8::1/124', state='absent'))
        result = self.execute_module()

    def test_nxos_l3_interface_rem_missing_ipv4_and_ipv6(self):
        self.mode = 'ethernet_noshut'
        set_module_args(dict(name='Ethernet1/1', ipv4='192.168.0.1/24', ipv6='2001:db8::1/124', state='absent'))
        result = self.execute_module()

    # Rem when existing
    def test_nxos_l3_interface_rem_existing_ipv4(self):
        self.mode = 'ethernet_noshut_ipv4_ipv6'
        set_module_args(dict(name='Ethernet1/1', ipv4='192.168.0.1/24', state='absent'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['interface Ethernet1/1', 'no ip address 192.168.0.1/24', 'exit'])

    def test_nxos_l3_interface_rem_existing_ipv4_on_e11(self):
        self.mode = 'ethernet_noshut_ipv4_ipv6'
        set_module_args(dict(name='et1/1', ipv4='192.168.0.1/24', state='absent'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['interface Ethernet1/1', 'no ip address 192.168.0.1/24', 'exit'])

    def test_nxos_l3_interface_rem_existing_ipv6(self):
        self.mode = 'ethernet_noshut_ipv4_ipv6'
        set_module_args(dict(name='Ethernet1/1', ipv6='2001:db8::1/124', state='absent'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['interface Ethernet1/1', 'no ipv6 address 2001:db8::1/124', 'exit'])

    def test_nxos_l3_interface_rem_existing_ipv4_and_ipv6(self):
        self.mode = 'ethernet_noshut_ipv4_ipv6'
        set_module_args(dict(name='Ethernet1/1', ipv4='192.168.0.1/24', ipv6='2001:db8::1/124', state='absent'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['interface Ethernet1/1', 'no ip address 192.168.0.1/24', 'no ipv6 address 2001:db8::1/124', 'exit'])

    # Rem when existing with multiple IPv6
    def test_nxos_l3_interface_multiple_ipv6_rem_first(self):
        self.mode = 'ethernet_noshut_multiple_ipv6'
        set_module_args(dict(name='Ethernet1/1', ipv6='2001:db8::1/124', state='absent'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['interface Ethernet1/1', 'no ipv6 address 2001:db8::1/124', 'exit'])

    def test_nxos_l3_interface_multiple_ipv6_rem_last(self):
        self.mode = 'ethernet_noshut_multiple_ipv6'
        set_module_args(dict(name='Ethernet1/1', ipv6='2001:db8:2::1/124', state='absent'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['interface Ethernet1/1', 'no ipv6 address 2001:db8:2::1/124', 'exit'])

    # Rem when missing with aggregate
    def test_nxos_l3_interface_rem_with_empty_aggregate(self):
        self.mode = 'ethernet_noshut_multiple_ipv6'
        set_module_args(dict(aggregate=[], state='absent'))
        result = self.execute_module()

    def test_nxos_l3_interface_rem_missing_with_aggregate(self):
        self.mode = 'ethernet_noshut_multiple_ipv6'
        set_module_args(dict(state='absent', aggregate=[
            dict(name='Ethernet1/1', ipv4='192.168.0.2/24', ipv6='2001:db8::2/124'),
            dict(name='Ethernet1/1', ipv6='2001:db8:1::2/124'),
            dict(name='Ethernet1/1', ipv6='2001:db8:2::2/124')]))
        result = self.execute_module()

    # Rem when existing with aggregate
    def test_nxos_l3_interface_rem_existing_with_aggregate(self):
        self.mode = 'ethernet_noshut_multiple_ipv6'
        set_module_args(dict(state='absent', aggregate=[
            dict(name='Ethernet1/1', ipv4='192.168.0.1/24', ipv6='2001:db8::1/124'),
            dict(name='Ethernet1/1', ipv6='2001:db8:1::1/124'),
            dict(name='Ethernet1/1', ipv6='2001:db8:2::1/124')]))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], [
            'interface Ethernet1/1', 'no ip address 192.168.0.1/24', 'no ipv6 address 2001:db8::1/124', 'exit',
            'interface Ethernet1/1', 'no ipv6 address 2001:db8:1::1/124', 'exit',
            'interface Ethernet1/1', 'no ipv6 address 2001:db8:2::1/124', 'exit'])

    # Add itf only
    def test_nxos_l3_interface_add_on_itf_only(self):
        self.mode = 'ethernet_noshut'
        set_module_args(dict(name='Ethernet1/1'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'], ['interface Ethernet1/1'])

    # Add unknown interface
    def test_nxos_l3_interface_add_on_unknown_itf(self):
        self.mode = 'ethernet_noshut'
        set_module_args(dict(name='Ethernet1/2', ipv4='192.168.0.1/24'))
        result = self.execute_module()
        self.assertEqual(result['warnings'], ['Unknown interface Ethernet1/2'])

    # Rem unknown interface
    def test_nxos_l3_interface_rem_on_unknown_itf(self):
        self.mode = 'ethernet_noshut'
        set_module_args(dict(name='Ethernet1/2', ipv4='192.168.0.1/24', state='absent'))
        result = self.execute_module()
        self.assertEqual(result['warnings'], ['Unknown interface Ethernet1/2'])



# (c) 2018, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

''' unit test template for ONTAP Ansible module '''

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch, Mock
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_lun_copy \
    import NetAppOntapLUNCopy as my_module  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None, parm1=None):
        ''' save arguments '''
        self.type = kind
        self.parm1 = parm1
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        if self.type == 'destination_vserver':
            xml = self.build_lun_info(self.parm1)
        self.xml_out = xml
        return xml

    @staticmethod
    def build_lun_info(data):
        ''' build xml data for lun-info '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = {
            'num-records': 1,
        }
        xml.translate_struct(attributes)
        return xml


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        self.mock_lun_copy = {
            'source_vserver': 'ansible',
            'destination_path': '/vol/test/test_copy_dest_dest_new_reviewd_new',
            'source_path': '/vol/test/test_copy_1',
            'destination_vserver': 'ansible',
            'state': 'present'
        }

    def mock_args(self):

        return {
            'source_vserver': self.mock_lun_copy['source_vserver'],
            'destination_path': self.mock_lun_copy['destination_path'],
            'source_path': self.mock_lun_copy['source_path'],
            'destination_vserver': self.mock_lun_copy['destination_vserver'],
            'state': self.mock_lun_copy['state'],
            'hostname': 'hostname',
            'username': 'username',
            'password': 'password',
        }
        # self.server = MockONTAPConnection()

    def get_lun_copy_mock_object(self, kind=None):
        """
        Helper method to return an na_ontap_lun_copy object
        :param kind: passes this param to MockONTAPConnection()
        :return: na_ontap_interface object
        """
        lun_copy_obj = my_module()
        lun_copy_obj.autosupport_log = Mock(return_value=None)
        if kind is None:
            lun_copy_obj.server = MockONTAPConnection()
        else:
            lun_copy_obj.server = MockONTAPConnection(kind=kind)
        return lun_copy_obj

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            my_module()
        print('Info: %s' % exc.value.args[0]['msg'])

    def test_create_error_missing_param(self):
        ''' Test if create throws an error if required param 'destination_vserver' is not specified'''
        data = self.mock_args()
        del data['destination_vserver']
        set_module_args(data)
        with pytest.raises(AnsibleFailJson) as exc:
            self.get_lun_copy_mock_object('lun_copy').copy_lun()
        msg = 'Error: Missing one or more required parameters for copying lun: ' \
              'destination_path, source_path, destination_path'
        expected = sorted(','.split(msg))
        received = sorted(','.split(exc.value.args[0]['msg']))
        assert expected == received

    def test_successful_copy(self):
        ''' Test successful create '''
        # data = self.mock_args()
        set_module_args(self.mock_args())
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_lun_copy_mock_object().apply()
        assert exc.value.args[0]['changed']

    def test_copy_idempotency(self):
        ''' Test create idempotency '''
        set_module_args(self.mock_args())
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_lun_copy_mock_object('destination_vserver').apply()
        assert not exc.value.args[0]['changed']



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_wanopt_profile
short_description: Configure WAN optimization profiles in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify wanopt feature and profile category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    wanopt_profile:
        description:
            - Configure WAN optimization profiles.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            auth-group:
                description:
                    - Optionally add an authentication group to restrict access to the WAN Optimization tunnel to peers in the authentication group. Source
                       wanopt.auth-group.name.
            cifs:
                description:
                    - Enable/disable CIFS (Windows sharing) WAN Optimization and configure CIFS WAN Optimization features.
                suboptions:
                    byte-caching:
                        description:
                            - Enable/disable byte-caching for HTTP. Byte caching reduces the amount of traffic by caching file data sent across the WAN and in
                               future serving if from the cache.
                        choices:
                            - enable
                            - disable
                    log-traffic:
                        description:
                            - Enable/disable logging.
                        choices:
                            - enable
                            - disable
                    port:
                        description:
                            - Single port number or port number range for CIFS. Only packets with a destination port number that matches this port number or
                               range are accepted by this profile.
                    prefer-chunking:
                        description:
                            - Select dynamic or fixed-size data chunking for HTTP WAN Optimization.
                        choices:
                            - dynamic
                            - fix
                    secure-tunnel:
                        description:
                            - Enable/disable securing the WAN Opt tunnel using SSL. Secure and non-secure tunnels use the same TCP port (7810).
                        choices:
                            - enable
                            - disable
                    status:
                        description:
                            - Enable/disable HTTP WAN Optimization.
                        choices:
                            - enable
                            - disable
                    tunnel-sharing:
                        description:
                            - Tunnel sharing mode for aggressive/non-aggressive and/or interactive/non-interactive protocols.
                        choices:
                            - private
                            - shared
                            - express-shared
            comments:
                description:
                    - Comment.
            ftp:
                description:
                    - Enable/disable FTP WAN Optimization and configure FTP WAN Optimization features.
                suboptions:
                    byte-caching:
                        description:
                            - Enable/disable byte-caching for HTTP. Byte caching reduces the amount of traffic by caching file data sent across the WAN and in
                               future serving if from the cache.
                        choices:
                            - enable
                            - disable
                    log-traffic:
                        description:
                            - Enable/disable logging.
                        choices:
                            - enable
                            - disable
                    port:
                        description:
                            - Single port number or port number range for FTP. Only packets with a destination port number that matches this port number or
                               range are accepted by this profile.
                    prefer-chunking:
                        description:
                            - Select dynamic or fixed-size data chunking for HTTP WAN Optimization.
                        choices:
                            - dynamic
                            - fix
                    secure-tunnel:
                        description:
                            - Enable/disable securing the WAN Opt tunnel using SSL. Secure and non-secure tunnels use the same TCP port (7810).
                        choices:
                            - enable
                            - disable
                    status:
                        description:
                            - Enable/disable HTTP WAN Optimization.
                        choices:
                            - enable
                            - disable
                    tunnel-sharing:
                        description:
                            - Tunnel sharing mode for aggressive/non-aggressive and/or interactive/non-interactive protocols.
                        choices:
                            - private
                            - shared
                            - express-shared
            http:
                description:
                    - Enable/disable HTTP WAN Optimization and configure HTTP WAN Optimization features.
                suboptions:
                    byte-caching:
                        description:
                            - Enable/disable byte-caching for HTTP. Byte caching reduces the amount of traffic by caching file data sent across the WAN and in
                               future serving if from the cache.
                        choices:
                            - enable
                            - disable
                    log-traffic:
                        description:
                            - Enable/disable logging.
                        choices:
                            - enable
                            - disable
                    port:
                        description:
                            - Single port number or port number range for HTTP. Only packets with a destination port number that matches this port number or
                               range are accepted by this profile.
                    prefer-chunking:
                        description:
                            - Select dynamic or fixed-size data chunking for HTTP WAN Optimization.
                        choices:
                            - dynamic
                            - fix
                    secure-tunnel:
                        description:
                            - Enable/disable securing the WAN Opt tunnel using SSL. Secure and non-secure tunnels use the same TCP port (7810).
                        choices:
                            - enable
                            - disable
                    ssl:
                        description:
                            - Enable/disable SSL/TLS offloading (hardware acceleration) for HTTPS traffic in this tunnel.
                        choices:
                            - enable
                            - disable
                    ssl-port:
                        description:
                            - Port on which to expect HTTPS traffic for SSL/TLS offloading.
                    status:
                        description:
                            - Enable/disable HTTP WAN Optimization.
                        choices:
                            - enable
                            - disable
                    tunnel-non-http:
                        description:
                            - Configure how to process non-HTTP traffic when a profile configured for HTTP traffic accepts a non-HTTP session. Can occur if an
                               application sends non-HTTP traffic using an HTTP destination port.
                        choices:
                            - enable
                            - disable
                    tunnel-sharing:
                        description:
                            - Tunnel sharing mode for aggressive/non-aggressive and/or interactive/non-interactive protocols.
                        choices:
                            - private
                            - shared
                            - express-shared
                    unknown-http-version:
                        description:
                            - How to handle HTTP sessions that do not comply with HTTP 0.9, 1.0, or 1.1.
                        choices:
                            - reject
                            - tunnel
                            - best-effort
            mapi:
                description:
                    - Enable/disable MAPI email WAN Optimization and configure MAPI WAN Optimization features.
                suboptions:
                    byte-caching:
                        description:
                            - Enable/disable byte-caching for HTTP. Byte caching reduces the amount of traffic by caching file data sent across the WAN and in
                               future serving if from the cache.
                        choices:
                            - enable
                            - disable
                    log-traffic:
                        description:
                            - Enable/disable logging.
                        choices:
                            - enable
                            - disable
                    port:
                        description:
                            - Single port number or port number range for MAPI. Only packets with a destination port number that matches this port number or
                               range are accepted by this profile.
                    secure-tunnel:
                        description:
                            - Enable/disable securing the WAN Opt tunnel using SSL. Secure and non-secure tunnels use the same TCP port (7810).
                        choices:
                            - enable
                            - disable
                    status:
                        description:
                            - Enable/disable HTTP WAN Optimization.
                        choices:
                            - enable
                            - disable
                    tunnel-sharing:
                        description:
                            - Tunnel sharing mode for aggressive/non-aggressive and/or interactive/non-interactive protocols.
                        choices:
                            - private
                            - shared
                            - express-shared
            name:
                description:
                    - Profile name.
                required: true
            tcp:
                description:
                    - Enable/disable TCP WAN Optimization and configure TCP WAN Optimization features.
                suboptions:
                    byte-caching:
                        description:
                            - Enable/disable byte-caching for HTTP. Byte caching reduces the amount of traffic by caching file data sent across the WAN and in
                               future serving if from the cache.
                        choices:
                            - enable
                            - disable
                    byte-caching-opt:
                        description:
                            - Select whether TCP byte-caching uses system memory only or both memory and disk space.
                        choices:
                            - mem-only
                            - mem-disk
                    log-traffic:
                        description:
                            - Enable/disable logging.
                        choices:
                            - enable
                            - disable
                    port:
                        description:
                            - Single port number or port number range for TCP. Only packets with a destination port number that matches this port number or
                               range are accepted by this profile.
                    secure-tunnel:
                        description:
                            - Enable/disable securing the WAN Opt tunnel using SSL. Secure and non-secure tunnels use the same TCP port (7810).
                        choices:
                            - enable
                            - disable
                    ssl:
                        description:
                            - Enable/disable SSL/TLS offloading.
                        choices:
                            - enable
                            - disable
                    ssl-port:
                        description:
                            - Port on which to expect HTTPS traffic for SSL/TLS offloading.
                    status:
                        description:
                            - Enable/disable HTTP WAN Optimization.
                        choices:
                            - enable
                            - disable
                    tunnel-sharing:
                        description:
                            - Tunnel sharing mode for aggressive/non-aggressive and/or interactive/non-interactive protocols.
                        choices:
                            - private
                            - shared
                            - express-shared
            transparent:
                description:
                    - Enable/disable transparent mode.
                choices:
                    - enable
                    - disable
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure WAN optimization profiles.
    fortios_wanopt_profile:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      wanopt_profile:
        state: "present"
        auth-group: "<your_own_value> (source wanopt.auth-group.name)"
        cifs:
            byte-caching: "enable"
            log-traffic: "enable"
            port: "7"
            prefer-chunking: "dynamic"
            secure-tunnel: "enable"
            status: "enable"
            tunnel-sharing: "private"
        comments: "<your_own_value>"
        ftp:
            byte-caching: "enable"
            log-traffic: "enable"
            port: "16"
            prefer-chunking: "dynamic"
            secure-tunnel: "enable"
            status: "enable"
            tunnel-sharing: "private"
        http:
            byte-caching: "enable"
            log-traffic: "enable"
            port: "24"
            prefer-chunking: "dynamic"
            secure-tunnel: "enable"
            ssl: "enable"
            ssl-port: "28"
            status: "enable"
            tunnel-non-http: "enable"
            tunnel-sharing: "private"
            unknown-http-version: "reject"
        mapi:
            byte-caching: "enable"
            log-traffic: "enable"
            port: "36"
            secure-tunnel: "enable"
            status: "enable"
            tunnel-sharing: "private"
        name: "default_name_40"
        tcp:
            byte-caching: "enable"
            byte-caching-opt: "mem-only"
            log-traffic: "enable"
            port: "<your_own_value>"
            secure-tunnel: "enable"
            ssl: "enable"
            ssl-port: "48"
            status: "enable"
            tunnel-sharing: "private"
        transparent: "enable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule


def login(data, fos):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_wanopt_profile_data(json):
    option_list = ['auth-group', 'cifs', 'comments',
                   'ftp', 'http', 'mapi',
                   'name', 'tcp', 'transparent']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def wanopt_profile(data, fos):
    vdom = data['vdom']
    wanopt_profile_data = data['wanopt_profile']
    filtered_data = filter_wanopt_profile_data(wanopt_profile_data)

    if wanopt_profile_data['state'] == "present":
        return fos.set('wanopt',
                       'profile',
                       data=filtered_data,
                       vdom=vdom)

    elif wanopt_profile_data['state'] == "absent":
        return fos.delete('wanopt',
                          'profile',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_wanopt(data, fos):
    login(data, fos)

    if data['wanopt_profile']:
        resp = wanopt_profile(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "wanopt_profile": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "auth-group": {"required": False, "type": "str"},
                "cifs": {"required": False, "type": "dict",
                         "options": {
                             "byte-caching": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                             "log-traffic": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                             "port": {"required": False, "type": "int"},
                             "prefer-chunking": {"required": False, "type": "str",
                                                 "choices": ["dynamic", "fix"]},
                             "secure-tunnel": {"required": False, "type": "str",
                                               "choices": ["enable", "disable"]},
                             "status": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                             "tunnel-sharing": {"required": False, "type": "str",
                                                "choices": ["private", "shared", "express-shared"]}
                         }},
                "comments": {"required": False, "type": "str"},
                "ftp": {"required": False, "type": "dict",
                        "options": {
                            "byte-caching": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                            "log-traffic": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                            "port": {"required": False, "type": "int"},
                            "prefer-chunking": {"required": False, "type": "str",
                                                "choices": ["dynamic", "fix"]},
                            "secure-tunnel": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                            "status": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                            "tunnel-sharing": {"required": False, "type": "str",
                                               "choices": ["private", "shared", "express-shared"]}
                        }},
                "http": {"required": False, "type": "dict",
                         "options": {
                             "byte-caching": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                             "log-traffic": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                             "port": {"required": False, "type": "int"},
                             "prefer-chunking": {"required": False, "type": "str",
                                                 "choices": ["dynamic", "fix"]},
                             "secure-tunnel": {"required": False, "type": "str",
                                               "choices": ["enable", "disable"]},
                             "ssl": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                             "ssl-port": {"required": False, "type": "int"},
                             "status": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                             "tunnel-non-http": {"required": False, "type": "str",
                                                 "choices": ["enable", "disable"]},
                             "tunnel-sharing": {"required": False, "type": "str",
                                                "choices": ["private", "shared", "express-shared"]},
                             "unknown-http-version": {"required": False, "type": "str",
                                                      "choices": ["reject", "tunnel", "best-effort"]}
                         }},
                "mapi": {"required": False, "type": "dict",
                         "options": {
                             "byte-caching": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                             "log-traffic": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                             "port": {"required": False, "type": "int"},
                             "secure-tunnel": {"required": False, "type": "str",
                                               "choices": ["enable", "disable"]},
                             "status": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                             "tunnel-sharing": {"required": False, "type": "str",
                                                "choices": ["private", "shared", "express-shared"]}
                         }},
                "name": {"required": True, "type": "str"},
                "tcp": {"required": False, "type": "dict",
                        "options": {
                            "byte-caching": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                            "byte-caching-opt": {"required": False, "type": "str",
                                                 "choices": ["mem-only", "mem-disk"]},
                            "log-traffic": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                            "port": {"required": False, "type": "str"},
                            "secure-tunnel": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                            "ssl": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                            "ssl-port": {"required": False, "type": "int"},
                            "status": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                            "tunnel-sharing": {"required": False, "type": "str",
                                               "choices": ["private", "shared", "express-shared"]}
                        }},
                "transparent": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_wanopt(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# Copyright: (c) 2018, F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import json
import pytest
import sys

if sys.version_info < (2, 7):
    pytestmark = pytest.mark.skip("F5 Ansible modules require Python >= 2.7")

from ansible.module_utils.basic import AnsibleModule

try:
    from library.modules.bigip_device_ha_group import ApiParameters
    from library.modules.bigip_device_ha_group import ModuleParameters
    from library.modules.bigip_device_ha_group import ModuleManager
    from library.modules.bigip_device_ha_group import ArgumentSpec

    # In Ansible 2.8, Ansible changed import paths.
    from test.units.compat import unittest
    from test.units.compat.mock import Mock
    from test.units.compat.mock import patch

    from test.units.modules.utils import set_module_args
except ImportError:
    from ansible.modules.network.f5.bigip_device_ha_group import ApiParameters
    from ansible.modules.network.f5.bigip_device_ha_group import ModuleParameters
    from ansible.modules.network.f5.bigip_device_ha_group import ModuleManager
    from ansible.modules.network.f5.bigip_device_ha_group import ArgumentSpec

    # Ansible 2.8 imports
    from units.compat import unittest
    from units.compat.mock import Mock
    from units.compat.mock import patch

    from units.modules.utils import set_module_args


fixture_path = os.path.join(os.path.dirname(__file__), 'fixtures')
fixture_data = {}


def load_fixture(name):
    path = os.path.join(fixture_path, name)

    if path in fixture_data:
        return fixture_data[path]

    with open(path) as f:
        data = f.read()

    try:
        data = json.loads(data)
    except Exception:
        pass

    fixture_data[path] = data
    return data


class TestParameters(unittest.TestCase):
    def test_module_parameters_v13(self):
        args = dict(
            name='foobar',
            description='baz',
            active_bonus=20,
            enable='yes',
            state='present',
            pools=[
                dict(
                    pool_name='fakepool',
                    attribute='percent-up-members',
                    weight=30,
                    minimum_threshold=2,
                    partition='Common'
                )
            ],
            trunks=[
                dict(
                    trunk_name='faketrunk',
                    attribute='percent-up-members',
                    weight=30,
                    minimum_threshold=2
                )
            ]
        )

        try:
            self.p1 = patch('library.modules.bigip_device_ha_group.tmos_version')
            self.m1 = self.p1.start()
            self.m1.return_value = '13.1.0'
        except Exception:
            self.p1 = patch('ansible.modules.network.f5.bigip_device_ha_group.tmos_version')
            self.m1 = self.p1.start()
            self.m1.return_value = '13.1.0'

        p = ModuleParameters(params=args)

        assert p.name == 'foobar'
        assert p.state == 'present'
        assert p.active_bonus == 20
        assert p.enabled is True
        assert p.pools == [{'name': '/Common/fakepool', 'attribute': 'percent-up-members',
                            'weight': 30, 'minimumThreshold': 2}]
        assert p.trunks == [{'name': 'faketrunk', 'attribute': 'percent-up-members',
                             'weight': 30, 'minimumThreshold': 2}]

        self.p1.stop()

    def test_module_parameters_v12(self):
        args = dict(
            name='foobar',
            description='baz',
            active_bonus=20,
            enable='yes',
            state='present',
            pools=[
                dict(
                    pool_name='fakepool',
                    attribute='percent-up-members',
                    weight=30,
                    minimum_threshold=2,
                    partition='Common'
                )
            ],
            trunks=[
                dict(
                    trunk_name='faketrunk',
                    attribute='percent-up-members',
                    weight=20,
                    minimum_threshold=1
                )
            ]
        )

        try:
            self.p1 = patch('library.modules.bigip_device_ha_group.tmos_version')
            self.m1 = self.p1.start()
            self.m1.return_value = '12.1.0'
        except Exception:
            self.p1 = patch('ansible.modules.network.f5.bigip_device_ha_group.tmos_version')
            self.m1 = self.p1.start()
            self.m1.return_value = '12.1.0'

        p = ModuleParameters(params=args)

        assert p.name == 'foobar'
        assert p.state == 'present'
        assert p.active_bonus == 20
        assert p.enabled is True
        assert p.pools == [{'name': '/Common/fakepool', 'attribute': 'percent-up-members',
                            'weight': 30, 'threshold': 2}]
        assert p.trunks == [{'name': 'faketrunk', 'attribute': 'percent-up-members',
                             'weight': 20, 'threshold': 1}]

        self.p1.stop()


class TestManager(unittest.TestCase):
    def setUp(self):
        self.spec = ArgumentSpec()

        try:
            self.p1 = patch('library.modules.bigip_device_ha_group.tmos_version')
            self.m1 = self.p1.start()
            self.m1.return_value = '13.1.0'
        except Exception:
            self.p1 = patch('ansible.modules.network.f5.bigip_device_ha_group.tmos_version')
            self.m1 = self.p1.start()
            self.m1.return_value = '13.1.0'

    def tearDown(self):
        self.p1.stop()

    def test_create_ha_group(self, *args):
        set_module_args(dict(
            name='fake_group',
            state='present',
            description='baz',
            active_bonus=20,
            enable='yes',
            pools=[
                dict(
                    pool_name='fakepool',
                    attribute='percent-up-members',
                    weight=30,
                    minimum_threshold=2,
                    partition='Common'
                )
            ],
            trunks=[
                dict(
                    trunk_name='faketrunk',
                    attribute='percent-up-members',
                    weight=20,
                    minimum_threshold=1
                )
            ],
            provider=dict(
                server='localhost',
                password='password',
                user='admin'
            )
        ))

        module = AnsibleModule(
            argument_spec=self.spec.argument_spec,
            supports_check_mode=self.spec.supports_check_mode
        )

        # Override methods to force specific logic in the module to happen
        mm = ModuleManager(module=module)
        mm.exists = Mock(return_value=False)
        mm.create_on_device = Mock(return_value=True)

        results = mm.exec_module()

        assert results['changed'] is True
        assert results['name'] == 'fake_group'
        assert results['description'] == 'baz'
        assert results['active_bonus'] == 20
        assert results['enable'] == 'yes'
        assert results['pools'] == [{'pool_name': '/Common/fakepool', 'attribute': 'percent-up-members',
                                     'weight': 30, 'minimum_threshold': 2}]
        assert results['trunks'] == [{'trunk_name': 'faketrunk', 'attribute': 'percent-up-members',
                                      'weight': 20, 'minimum_threshold': 1}]



#
# (c) 2018 Extreme Networks Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import re

from ansible.errors import AnsibleConnectionFailure
from ansible.plugins.terminal import TerminalBase


class TerminalModule(TerminalBase):

    terminal_stdout_re = [
        re.compile(br"([\r\n]|(\x1b\[\?7h))[\w\+\-\.:\/\[\]]+(?:\([^\)]+\)){0,3}(?:[>#]) ?$")
    ]

    terminal_stderr_re = [
        re.compile(br"% ?Error"),
        # re.compile(br"^% \w+", re.M),
        re.compile(br"% ?Bad secret"),
        re.compile(br"[\r\n%] Bad passwords"),
        re.compile(br"invalid input", re.I),
        re.compile(br"(?:incomplete|ambiguous) command", re.I),
        re.compile(br"connection timed out", re.I),
        re.compile(br"[^\r\n]+ not found"),
        re.compile(br"'[^']' +returned error code: ?\d+"),
        re.compile(br"Bad mask", re.I),
        re.compile(br"% ?(\S+) ?overlaps with ?(\S+)", re.I),
        re.compile(br"[%\S] ?Informational: ?[\s]+", re.I),
        re.compile(br"syntax error: unknown argument.", re.I)
    ]

    def on_open_shell(self):
        try:
            self._exec_cli_command(u'terminal length 0')
        except AnsibleConnectionFailure:
            raise AnsibleConnectionFailure('unable to set terminal parameters')



# -*- coding: utf-8 -*-

# Copyright: (c) 2018,  Red Hat | Ansible
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Options for authenticating with the API.


class ModuleDocFragment(object):

    DOCUMENTATION = r'''
options:
  host:
    description:
    - Provide a URL for accessing the API. Can also be specified via K8S_AUTH_HOST environment variable.
    type: str
  api_key:
    description:
    - Token used to authenticate with the API. Can also be specified via K8S_AUTH_API_KEY environment variable.
    type: str
  kubeconfig:
    description:
    - Path to an existing Kubernetes config file. If not provided, and no other connection
      options are provided, the openshift client will attempt to load the default
      configuration file from I(~/.kube/config.json). Can also be specified via K8S_AUTH_KUBECONFIG environment
      variable.
    type: path
  context:
    description:
    - The name of a context found in the config file. Can also be specified via K8S_AUTH_CONTEXT environment variable.
    type: str
  username:
    description:
    - Provide a username for authenticating with the API. Can also be specified via K8S_AUTH_USERNAME environment
      variable.
    - Please note that this only works with clusters configured to use HTTP Basic Auth. If your cluster has a
      different form of authentication (e.g. OAuth2 in OpenShift), this option will not work as expected and you
      should look into the C(k8s_auth) module, as that might do what you need.
    type: str
  password:
    description:
    - Provide a password for authenticating with the API. Can also be specified via K8S_AUTH_PASSWORD environment
      variable.
    - Please read the description of the C(username) option for a discussion of when this option is applicable.
    type: str
  client_cert:
    description:
    - Path to a certificate used to authenticate with the API. Can also be specified via K8S_AUTH_CERT_FILE environment
      variable.
    type: path
    aliases: [ cert_file ]
  client_key:
    description:
    - Path to a key file used to authenticate with the API. Can also be specified via K8S_AUTH_KEY_FILE environment
      variable.
    type: path
    aliases: [ key_file ]
  ca_cert:
    description:
    - Path to a CA certificate used to authenticate with the API. The full certificate chain must be provided to
      avoid certificate validation errors. Can also be specified via K8S_AUTH_SSL_CA_CERT environment variable.
    type: path
    aliases: [ ssl_ca_cert ]
  validate_certs:
    description:
    - Whether or not to verify the API server's SSL certificates. Can also be specified via K8S_AUTH_VERIFY_SSL
      environment variable.
    type: bool
    aliases: [ verify_ssl ]
notes:
  - "The OpenShift Python client wraps the K8s Python client, providing full access to
    all of the APIS and models available on both platforms. For API version details and
    additional information visit https://github.com/openshift/openshift-restclient-python"
  - "To avoid SSL certificate validation errors when C(validate_certs) is I(True), the full
    certificate chain for the API server must be provided via C(ca_cert) or in the
    kubeconfig file."
'''



# -*- coding: utf-8 -*-

# (c) 2019, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from units.compat.mock import patch
from ansible.modules.network.asa import asa_og
from units.modules.utils import set_module_args
from .asa_module import TestAsaModule, load_fixture


class TestAsaOgModule(TestAsaModule):

    module = asa_og

    def setUp(self):
        super(TestAsaOgModule, self).setUp()

        self.mock_get_config = patch('ansible.modules.network.asa.asa_og.get_config')
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch('ansible.modules.network.asa.asa_og.load_config')
        self.load_config = self.mock_load_config.start()

        self.mock_get_connection = patch('ansible.module_utils.network.asa.asa.get_connection')
        self.get_connection = self.mock_get_connection.start()

    def tearDown(self):
        super(TestAsaOgModule, self).tearDown()
        self.mock_get_config.stop()
        self.mock_load_config.stop()

    def load_fixtures(self, commands=None):
        self.get_config.return_value = load_fixture('asa_og_config.cfg').strip()
        self.load_config.return_value = dict(diff=None, session='session')

    def test_asa_og_idempotent(self):
        set_module_args(dict(
            name='test_nets',
            group_type='network-object',
            host_ip=['8.8.8.8'],
            ip_mask=['192.168.0.0 255.255.0.0'],
            group_object=['awx_lon'],
            description='ansible_test object-group description',
            state='present'
        ))
        commands = []
        self.execute_module(changed=False, commands=commands)

    def test_asa_og_add(self):
        set_module_args(dict(
            name='test_nets',
            group_type='network-object',
            host_ip=['8.8.8.8', '8.8.4.4'],
            ip_mask=['192.168.0.0 255.255.0.0', '10.0.0.0 255.255.255.0'],
            group_object=['awx_lon', 'awx_ams'],
            description='ansible_test object-group description',
            state='present'
        ))
        commands = [
            'object-group network test_nets',
            'network-object host 8.8.4.4',
            'network-object 10.0.0.0 255.255.255.0',
            'group-object awx_ams'
        ]
        self.execute_module(changed=True, commands=commands)

    def test_asa_og_replace(self):
        set_module_args(dict(
            name='test_nets',
            group_type='network-object',
            host_ip=['8.8.4.4'],
            ip_mask=['10.0.0.0 255.255.255.0'],
            group_object=['awx_ams'],
            description='ansible_test custom description',
            state='replace'
        ))
        commands = [
            'object-group network test_nets',
            'description ansible_test custom description',
            'no network-object host 8.8.8.8',
            'network-object host 8.8.4.4',
            'no network-object 192.168.0.0 255.255.0.0',
            'network-object 10.0.0.0 255.255.255.0',
            'no group-object awx_lon',
            'group-object awx_ams'
        ]
        self.execute_module(changed=True, commands=commands)

    def test_asa_og_remove(self):
        set_module_args(dict(
            name='test_nets',
            group_type='network-object',
            host_ip=['8.8.8.8'],
            group_object=['awx_lon'],
            state='absent'
        ))
        commands = [
            'object-group network test_nets',
            'no network-object host 8.8.8.8',
            'no group-object awx_lon'
        ]
        self.execute_module(changed=True, commands=commands)



import pytest
import unittest

import ansible.modules.cloud.amazon.ec2_vpc_nat_gateway as ng


boto3 = pytest.importorskip("boto3")
botocore = pytest.importorskip("botocore")

aws_region = 'us-west-2'


class AnsibleEc2VpcNatGatewayFunctions(unittest.TestCase):

    def test_get_nat_gateways(self):
        client = boto3.client('ec2', region_name=aws_region)
        success, err_msg, stream = (
            ng.get_nat_gateways(client, 'subnet-123456789', check_mode=True)
        )
        should_return = ng.DRY_RUN_GATEWAYS
        self.assertTrue(success)
        self.assertEqual(stream, should_return)

    def test_get_nat_gateways_no_gateways_found(self):
        client = boto3.client('ec2', region_name=aws_region)
        success, err_msg, stream = (
            ng.get_nat_gateways(client, 'subnet-1234567', check_mode=True)
        )
        self.assertTrue(success)
        self.assertEqual(stream, [])

    def test_wait_for_status(self):
        client = boto3.client('ec2', region_name=aws_region)
        success, err_msg, gws = (
            ng.wait_for_status(
                client, 5, 'nat-123456789', 'available', check_mode=True
            )
        )
        should_return = ng.DRY_RUN_GATEWAYS[0]
        self.assertTrue(success)
        self.assertEqual(gws, should_return)

    def test_wait_for_status_to_timeout(self):
        client = boto3.client('ec2', region_name=aws_region)
        success, err_msg, gws = (
            ng.wait_for_status(
                client, 2, 'nat-12345678', 'available', check_mode=True
            )
        )
        self.assertFalse(success)
        self.assertEqual(gws, {})

    def test_gateway_in_subnet_exists_with_allocation_id(self):
        client = boto3.client('ec2', region_name=aws_region)
        gws, err_msg = (
            ng.gateway_in_subnet_exists(
                client, 'subnet-123456789', 'eipalloc-1234567', check_mode=True
            )
        )
        should_return = ng.DRY_RUN_GATEWAYS
        self.assertEqual(gws, should_return)

    def test_gateway_in_subnet_exists_with_allocation_id_does_not_exist(self):
        client = boto3.client('ec2', region_name=aws_region)
        gws, err_msg = (
            ng.gateway_in_subnet_exists(
                client, 'subnet-123456789', 'eipalloc-123', check_mode=True
            )
        )
        should_return = list()
        self.assertEqual(gws, should_return)

    def test_gateway_in_subnet_exists_without_allocation_id(self):
        client = boto3.client('ec2', region_name=aws_region)
        gws, err_msg = (
            ng.gateway_in_subnet_exists(
                client, 'subnet-123456789', check_mode=True
            )
        )
        should_return = ng.DRY_RUN_GATEWAYS
        self.assertEqual(gws, should_return)

    def test_get_eip_allocation_id_by_address(self):
        client = boto3.client('ec2', region_name=aws_region)
        allocation_id, _ = (
            ng.get_eip_allocation_id_by_address(
                client, '55.55.55.55', check_mode=True
            )
        )
        should_return = 'eipalloc-1234567'
        self.assertEqual(allocation_id, should_return)

    def test_get_eip_allocation_id_by_address_does_not_exist(self):
        client = boto3.client('ec2', region_name=aws_region)
        allocation_id, err_msg = (
            ng.get_eip_allocation_id_by_address(
                client, '52.52.52.52', check_mode=True
            )
        )
        self.assertEqual(err_msg, 'EIP 52.52.52.52 does not exist')
        self.assertTrue(allocation_id is None)

    def test_allocate_eip_address(self):
        client = boto3.client('ec2', region_name=aws_region)
        success, err_msg, eip_id = (
            ng.allocate_eip_address(
                client, check_mode=True
            )
        )
        self.assertTrue(success)

    def test_release_address(self):
        client = boto3.client('ec2', region_name=aws_region)
        success, _ = (
            ng.release_address(
                client, 'eipalloc-1234567', check_mode=True
            )
        )
        self.assertTrue(success)

    def test_create(self):
        client = boto3.client('ec2', region_name=aws_region)
        success, changed, err_msg, results = (
            ng.create(
                client, 'subnet-123456', 'eipalloc-1234567', check_mode=True
            )
        )
        self.assertTrue(success)
        self.assertTrue(changed)

    def test_pre_create(self):
        client = boto3.client('ec2', region_name=aws_region)
        success, changed, err_msg, results = (
            ng.pre_create(
                client, 'subnet-123456', check_mode=True
            )
        )
        self.assertTrue(success)
        self.assertTrue(changed)

    def test_pre_create_idemptotent_with_allocation_id(self):
        client = boto3.client('ec2', region_name=aws_region)
        success, changed, err_msg, results = (
            ng.pre_create(
                client, 'subnet-123456789', allocation_id='eipalloc-1234567', check_mode=True
            )
        )
        self.assertTrue(success)
        self.assertFalse(changed)

    def test_pre_create_idemptotent_with_eip_address(self):
        client = boto3.client('ec2', region_name=aws_region)
        success, changed, err_msg, results = (
            ng.pre_create(
                client, 'subnet-123456789', eip_address='55.55.55.55', check_mode=True
            )
        )
        self.assertTrue(success)
        self.assertFalse(changed)

    def test_pre_create_idemptotent_if_exist_do_not_create(self):
        client = boto3.client('ec2', region_name=aws_region)
        success, changed, err_msg, results = (
            ng.pre_create(
                client, 'subnet-123456789', if_exist_do_not_create=True, check_mode=True
            )
        )
        self.assertTrue(success)
        self.assertFalse(changed)

    def test_delete(self):
        client = boto3.client('ec2', region_name=aws_region)
        success, changed, err_msg, _ = (
            ng.remove(
                client, 'nat-123456789', check_mode=True
            )
        )
        self.assertTrue(success)
        self.assertTrue(changed)

    def test_delete_and_release_ip(self):
        client = boto3.client('ec2', region_name=aws_region)
        success, changed, err_msg, _ = (
            ng.remove(
                client, 'nat-123456789', release_eip=True, check_mode=True
            )
        )
        self.assertTrue(success)
        self.assertTrue(changed)

    def test_delete_if_does_not_exist(self):
        client = boto3.client('ec2', region_name=aws_region)
        success, changed, err_msg, _ = (
            ng.remove(
                client, 'nat-12345', check_mode=True
            )
        )
        self.assertFalse(success)
        self.assertFalse(changed)



# (c) 2018, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

''' unit tests ONTAP Ansible module: na_ontap_cluster '''

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch, Mock
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_cluster \
    import NetAppONTAPCluster as my_module  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None):
        ''' save arguments '''
        self.type = kind
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        if self.type == 'cluster':
            xml = self.build_cluster_info()
        elif self.type == 'cluster_fail':
            raise netapp_utils.zapi.NaApiError(code='TEST', message="This exception is from the unit test")
        self.xml_out = xml
        return xml

    def autosupport_log(self):
        ''' mock autosupport log'''
        return None

    @staticmethod
    def build_cluster_info():
        ''' build xml data for cluster-info '''
        xml = netapp_utils.zapi.NaElement('xml')
        data = {'license-v2-status': {'package': 'cifs', 'method': 'site'}}
        xml.translate_struct(data)
        print(xml.to_string())
        return xml


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        self.server = MockONTAPConnection()
        self.use_vsim = False

    def set_default_args(self):
        if self.use_vsim:
            hostname = '10.193.77.37'
            username = 'admin'
            password = 'netapp1!'
            license_package = 'CIFS'
            node_serial_number = '123'
            license_code = 'AAA'
            cluster_name = 'abc'
        else:
            hostname = '10.193.77.37'
            username = 'admin'
            password = 'netapp1!'
            license_package = 'CIFS'
            node_serial_number = '123'
            cluster_ip_address = '0.0.0.0'
            license_code = 'AAA'
            cluster_name = 'abc'
        return dict({
            'hostname': hostname,
            'username': username,
            'password': password,
            'license_package': license_package,
            'node_serial_number': node_serial_number,
            'license_code': license_code,
            'cluster_name': cluster_name,
            'cluster_ip_address': cluster_ip_address
        })

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            my_module()
        print('Info: %s' % exc.value.args[0]['msg'])

    def test_ensure_license_get_called(self):
        ''' fetching details of license '''
        set_module_args(self.set_default_args())
        my_obj = my_module()
        my_obj.server = self.server
        license_get = my_obj.get_licensing_status()
        print('Info: test_license_get: %s' % repr(license_get))
        assert not bool(license_get)

    def test_ensure_apply_for_cluster_called(self):
        ''' creating license and checking idempotency '''
        module_args = {}
        module_args.update(self.set_default_args())
        set_module_args(module_args)
        my_obj = my_module()
        my_obj.autosupport_log = Mock(return_value=None)
        if not self.use_vsim:
            my_obj.server = self.server
        with pytest.raises(AnsibleExitJson) as exc:
            my_obj.apply()
        print('Info: test_cluster_apply: %s' % repr(exc.value))
        assert exc.value.args[0]['changed']
        if not self.use_vsim:
            my_obj.server = MockONTAPConnection('cluster')
        with pytest.raises(AnsibleExitJson) as exc:
            my_obj.apply()
        print('Info: test_cluster_apply: %s' % repr(exc.value))
        assert exc.value.args[0]['changed']

    @patch('ansible.modules.storage.netapp.na_ontap_cluster.NetAppONTAPCluster.create_cluster')
    def test_cluster_create_called(self, cluster_create):
        ''' creating cluster'''
        module_args = {}
        module_args.update(self.set_default_args())
        set_module_args(module_args)
        my_obj = my_module()
        my_obj.autosupport_log = Mock(return_value=None)
        if not self.use_vsim:
            my_obj.server = MockONTAPConnection('cluster')
        with pytest.raises(AnsibleExitJson) as exc:
            my_obj.apply()
        print('Info: test_cluster_apply: %s' % repr(exc.value))
        cluster_create.assert_called_with()

    def test_if_all_methods_catch_exception(self):
        module_args = {}
        module_args.update(self.set_default_args())
        set_module_args(module_args)
        my_obj = my_module()
        if not self.use_vsim:
            my_obj.server = MockONTAPConnection('cluster_fail')
        with pytest.raises(AnsibleFailJson) as exc:
            my_obj.get_licensing_status()
        assert 'Error checking license status' in exc.value.args[0]['msg']
        with pytest.raises(AnsibleFailJson) as exc:
            my_obj.create_cluster()
        assert 'Error creating cluster' in exc.value.args[0]['msg']
        with pytest.raises(AnsibleFailJson) as exc:
            my_obj.cluster_join()
        assert 'Error adding node to cluster' in exc.value.args[0]['msg']
        with pytest.raises(AnsibleFailJson) as exc:
            my_obj.license_v2_add()
        assert 'Error adding license' in exc.value.args[0]['msg']
        with pytest.raises(AnsibleFailJson) as exc:
            my_obj.license_v2_delete()
            assert 'Error deleting license' in exc.value.args[0]['msg']



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2015, Heyo
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# this is a windows documentation stub.  actual code lives in the .ps1
# file of the same name

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: win_nssm
version_added: "2.0"
short_description: Install a service using NSSM
description:
    - Install a Windows service using the NSSM wrapper.
    - NSSM is a service helper which doesn't suck. See U(https://nssm.cc/) for more information.
requirements:
    - "nssm >= 2.24.0 # (install via M(win_chocolatey)) C(win_chocolatey: name=nssm)"
options:
  name:
    description:
      - Name of the service to operate on.
    type: str
    required: true
  state:
    description:
      - State of the service on the system.
      - Values C(started), C(stopped), and C(restarted) are deprecated since v2.8,
        please use the M(win_service) module instead to start, stop or restart the service.
    type: str
    choices: [ absent, present, started, stopped, restarted ]
    default: present
  application:
    description:
      - The application binary to run as a service
      - Required when I(state) is C(present), C(started), C(stopped), or C(restarted).
    type: path
  executable:
    description:
    - The location of the NSSM utility (in case it is not located in your PATH).
    type: path
    default: nssm.exe
    version_added: "2.8.0"
  description:
    description:
      - The description to set for the service.
    type: str
    version_added: "2.8.0"
  display_name:
    description:
      - The display name to set for the service.
    type: str
    version_added: "2.8.0"
  working_directory:
    version_added: "2.8.0"
    description:
      - The working directory to run the service executable from (defaults to the directory containing the application binary)
    type: path
    aliases: [ app_directory, chdir ]
  stdout_file:
    description:
      - Path to receive output.
    type: path
  stderr_file:
    description:
      - Path to receive error output.
    type: path
  app_parameters:
    description:
      - A string representing a dictionary of parameters to be passed to the application when it starts.
      - DEPRECATED since v2.8, please use I(arguments) instead.
      - This is mutually exclusive with I(arguments).
    type: str
  arguments:
    description:
      - Parameters to be passed to the application when it starts.
      - This can be either a simple string or a list.
      - This parameter was renamed from I(app_parameters_free_form) in 2.8.
      - This is mutually exclusive with I(app_parameters).
    aliases: [ app_parameters_free_form ]
    type: str
    version_added: "2.3"
  dependencies:
    description:
      - Service dependencies that has to be started to trigger startup, separated by comma.
      - DEPRECATED since v2.8, please use the M(win_service) module instead.
    type: list
  user:
    description:
      - User to be used for service startup.
      - DEPRECATED since v2.8, please use the M(win_service) module instead.
    type: str
  password:
    description:
      - Password to be used for service startup.
      - DEPRECATED since v2.8, please use the M(win_service) module instead.
    type: str
  start_mode:
    description:
      - If C(auto) is selected, the service will start at bootup.
      - C(delayed) causes a delayed but automatic start after boot (added in version 2.5).
      - C(manual) means that the service will start only when another service needs it.
      - C(disabled) means that the service will stay off, regardless if it is needed or not.
      - DEPRECATED since v2.8, please use the M(win_service) module instead.
    type: str
    choices: [ auto, delayed, disabled, manual ]
    default: auto
seealso:
  - module: win_service
notes:
  - The service will NOT be started after its creation when C(state=present).
  - Once the service is created, you can use the M(win_service) module to start it or configure
    some additionals properties, such as its startup type, dependencies, service account, and so on.
author:
  - Adam Keech (@smadam813)
  - George Frank (@georgefrank)
  - Hans-Joachim Kliemeck (@h0nIg)
  - Michael Wild (@themiwi)
  - Kevin Subileau (@ksubileau)
'''

EXAMPLES = r'''
- name: Install the foo service
  win_nssm:
    name: foo
    application: C:\windows\foo.exe

# This will yield the following command: C:\windows\foo.exe bar "true"
- name: Install the Consul service with a list of parameters
  win_nssm:
    name: Consul
    application: C:\consul\consul.exe
    arguments:
      - agent
      - -config-dir=C:\consul\config

# This is strictly equivalent to the previous example
- name: Install the Consul service with an arbitrary string of parameters
  win_nssm:
    name: Consul
    application: C:\consul\consul.exe
    arguments: agent -config-dir=C:\consul\config


# Install the foo service, and then configure and start it with win_service
- name: Install the foo service, redirecting stdout and stderr to the same file
  win_nssm:
    name: foo
    application: C:\windows\foo.exe
    stdout_file: C:\windows\foo.log
    stderr_file: C:\windows\foo.log

- name: Configure and start the foo service using win_service
  win_service:
    name: foo
    dependencies: [ adf, tcpip ]
    user: foouser
    password: secret
    start_mode: manual
    state: started

- name: Remove the foo service
  win_nssm:
    name: foo
    state: absent
'''



import pytest


class APIErrorMock(Exception):
    def __init__(self, message, response=None, explanation=None):
        self.message = message
        self.response = response
        self.explanation = explanation


@pytest.fixture(autouse=True)
def docker_module_mock(mocker):
    docker_module_mock = mocker.MagicMock()
    docker_utils_module_mock = mocker.MagicMock()
    docker_errors_module_mock = mocker.MagicMock()
    docker_errors_module_mock.APIError = APIErrorMock
    mock_modules = {
        'docker': docker_module_mock,
        'docker.utils': docker_utils_module_mock,
        'docker.errors': docker_errors_module_mock,
    }
    return mocker.patch.dict('sys.modules', **mock_modules)


@pytest.fixture(autouse=True)
def docker_swarm_service():
    from ansible.modules.cloud.docker import docker_swarm_service

    return docker_swarm_service


def test_retry_on_out_of_sequence_error(mocker, docker_swarm_service):
    run_mock = mocker.MagicMock(
        side_effect=APIErrorMock(
            message='',
            response=None,
            explanation='rpc error: code = Unknown desc = update out of sequence',
        )
    )
    manager = docker_swarm_service.DockerServiceManager(client=None)
    manager.run = run_mock
    with pytest.raises(APIErrorMock):
        manager.run_safe()
    assert run_mock.call_count == 3


def test_no_retry_on_general_api_error(mocker, docker_swarm_service):
    run_mock = mocker.MagicMock(
        side_effect=APIErrorMock(message='', response=None, explanation='some error')
    )
    manager = docker_swarm_service.DockerServiceManager(client=None)
    manager.run = run_mock
    with pytest.raises(APIErrorMock):
        manager.run_safe()
    assert run_mock.call_count == 1


def test_get_docker_environment(mocker, docker_swarm_service):
    env_file_result = {'TEST1': 'A', 'TEST2': 'B', 'TEST3': 'C'}
    env_dict = {'TEST3': 'CC', 'TEST4': 'D'}
    env_string = "TEST3=CC,TEST4=D"

    env_list = ['TEST3=CC', 'TEST4=D']
    expected_result = sorted(['TEST1=A', 'TEST2=B', 'TEST3=CC', 'TEST4=D'])
    mocker.patch.object(
        docker_swarm_service, 'parse_env_file', return_value=env_file_result
    )
    mocker.patch.object(
        docker_swarm_service,
        'format_environment',
        side_effect=lambda d: ['{0}={1}'.format(key, value) for key, value in d.items()],
    )
    # Test with env dict and file
    result = docker_swarm_service.get_docker_environment(
        env_dict, env_files=['dummypath']
    )
    assert result == expected_result
    # Test with env list and file
    result = docker_swarm_service.get_docker_environment(
        env_list,
        env_files=['dummypath']
    )
    assert result == expected_result
    # Test with env string and file
    result = docker_swarm_service.get_docker_environment(
        env_string, env_files=['dummypath']
    )
    assert result == expected_result

    assert result == expected_result
    # Test with empty env
    result = docker_swarm_service.get_docker_environment(
        [], env_files=None
    )
    assert result == []
    # Test with empty env_files
    result = docker_swarm_service.get_docker_environment(
        None, env_files=[]
    )
    assert result == []


def test_get_nanoseconds_from_raw_option(docker_swarm_service):
    value = docker_swarm_service.get_nanoseconds_from_raw_option('test', None)
    assert value is None

    value = docker_swarm_service.get_nanoseconds_from_raw_option('test', '1m30s535ms')
    assert value == 90535000000

    value = docker_swarm_service.get_nanoseconds_from_raw_option('test', 10000000000)
    assert value == 10000000000

    with pytest.raises(ValueError):
        docker_swarm_service.get_nanoseconds_from_raw_option('test', [])


def test_has_dict_changed(docker_swarm_service):
    assert not docker_swarm_service.has_dict_changed(
        {"a": 1},
        {"a": 1},
    )
    assert not docker_swarm_service.has_dict_changed(
        {"a": 1},
        {"a": 1, "b": 2}
    )
    assert docker_swarm_service.has_dict_changed(
        {"a": 1},
        {"a": 2, "b": 2}
    )
    assert docker_swarm_service.has_dict_changed(
        {"a": 1, "b": 1},
        {"a": 1}
    )
    assert not docker_swarm_service.has_dict_changed(
        None,
        {"a": 2, "b": 2}
    )
    assert docker_swarm_service.has_dict_changed(
        {},
        {"a": 2, "b": 2}
    )
    assert docker_swarm_service.has_dict_changed(
        {"a": 1},
        {}
    )
    assert docker_swarm_service.has_dict_changed(
        {"a": 1},
        None
    )
    assert not docker_swarm_service.has_dict_changed(
        {},
        {}
    )
    assert not docker_swarm_service.has_dict_changed(
        None,
        None
    )
    assert not docker_swarm_service.has_dict_changed(
        {},
        None
    )
    assert not docker_swarm_service.has_dict_changed(
        None,
        {}
    )


def test_has_list_of_dicts_changed(docker_swarm_service):
    assert docker_swarm_service.has_list_of_dicts_changed(
        [
            {"a": 1},
            {"b": 1}
        ],
        [
            {"a": 1}
        ]
    )
    assert docker_swarm_service.has_list_of_dicts_changed(
        [
            {"a": 1},
        ],
        [
            {"a": 1},
            {"b": 1},
        ]
    )
    assert not docker_swarm_service.has_list_of_dicts_changed(
        [
            {"a": 1},
            {"b": 1},
        ],
        [
            {"a": 1},
            {"b": 1}
        ]
    )
    assert not docker_swarm_service.has_list_of_dicts_changed(
        None,
        [
            {"b": 1},
            {"a": 1}
        ]
    )
    assert docker_swarm_service.has_list_of_dicts_changed(
        [],
        [
            {"b": 1},
            {"a": 1}
        ]
    )
    assert not docker_swarm_service.has_list_of_dicts_changed(
        None,
        None
    )
    assert not docker_swarm_service.has_list_of_dicts_changed(
        [],
        None
    )
    assert not docker_swarm_service.has_list_of_dicts_changed(
        None,
        []
    )
    assert not docker_swarm_service.has_list_of_dicts_changed(
        [
            {"src": 1, "dst": 2},
            {"src": 1, "dst": 2, "protocol": "udp"},
        ],
        [
            {"src": 1, "dst": 2, "protocol": "tcp"},
            {"src": 1, "dst": 2, "protocol": "udp"},
        ]
    )
    assert not docker_swarm_service.has_list_of_dicts_changed(
        [
            {"src": 1, "dst": 2, "protocol": "udp"},
            {"src": 1, "dst": 3, "protocol": "tcp"},
        ],
        [
            {"src": 1, "dst": 2, "protocol": "udp"},
            {"src": 1, "dst": 3, "protocol": "tcp"},
        ]
    )
    assert docker_swarm_service.has_list_of_dicts_changed(
        [
            {"src": 1, "dst": 2, "protocol": "udp"},
            {"src": 1, "dst": 2},
            {"src": 3, "dst": 4},
        ],
        [
            {"src": 1, "dst": 3, "protocol": "udp"},
            {"src": 1, "dst": 2, "protocol": "tcp"},
            {"src": 3, "dst": 4, "protocol": "tcp"},
        ]
    )
    assert docker_swarm_service.has_list_of_dicts_changed(
        [
            {"src": 1, "dst": 3, "protocol": "tcp"},
            {"src": 1, "dst": 2, "protocol": "udp"},
        ],
        [
            {"src": 1, "dst": 2, "protocol": "tcp"},
            {"src": 1, "dst": 2, "protocol": "udp"},
        ]
    )
    assert docker_swarm_service.has_list_of_dicts_changed(
        [
            {"src": 1, "dst": 2, "protocol": "udp"},
            {"src": 1, "dst": 2, "protocol": "tcp", "extra": {"test": "foo"}},
        ],
        [
            {"src": 1, "dst": 2, "protocol": "udp"},
            {"src": 1, "dst": 2, "protocol": "tcp"},
        ]
    )
